/*
	fancyfileselection.cc
*/

#include <gtk--/box.h>
#include <gtk--/fileselection.h>
#include <gtk--/frame.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/menu.h>
#include <gtk--/optionmenu.h>
#include <sstream>
#include "fancyfileselection.h"
#include "lang.h"
#include "string.h"


static struct {
	const char *name, *extension;
} file_formats[] = {
	{ "GraphThing", ".gt" },
	{ "GraphML", ".graphml" },
	{ "Graphviz", ".dot" },
	{ "Maple", ".mws" }
};

void FancyFileSelection::ok_click ()
{
        cancelled = false;
        Gtk::Main::quit ();
}

Gtk::OptionMenu *FancyFileSelection::genFileTypeMenu ()
{
	using namespace Gtk::Menu_Helpers;

	Gtk::OptionMenu *opt_menu = manage (new Gtk::OptionMenu ());
	Gtk::Menu *menu = manage (new Gtk::Menu ());
	MenuList &ml = menu->items ();

	int num = sizeof (file_formats) / sizeof (file_formats[0]);
	for (int i = 0; i < num; ++i) {
		std::ostringstream full;
		full << file_formats[i].name << " (*"
				<< file_formats[i].extension << ")";
		ml.push_back (MenuElem (full.str (),
			SigC::bind (SigC::slot (this,
				&FancyFileSelection::setFileType),
				file_formats[i].name)));
	}

	opt_menu->set_menu (menu);

	return opt_menu;
}

int FancyFileSelection::delete_event_impl (GdkEventAny *ev)
{
	Gtk::Main::quit ();
	return false;
}

FancyFileSelection::FancyFileSelection (const String &title, const String &mask,
					bool action_buttons, bool file_types)
				: Gtk::FileSelection (title.std_str ())
{
	complete (mask.std_str ());
	set_modal (true);

	if (file_types) {
		filetype = "GraphThing";
		Gtk::HBox *hbox = manage (new Gtk::HBox ());
		Gtk::Label *lbl = manage (new Gtk::Label (_("Format:")));
		hbox->pack_start (*lbl, false, false, 10);
		hbox->pack_start (*genFileTypeMenu (), true, true, 10);
		Gtk::Frame *frame = manage (new Gtk::Frame (_("Options")));
		frame->add (*hbox);
		get_action_area ()->pack_start (*frame, true, true, 10);
	}

	get_ok_button ()->clicked.connect
			(SigC::slot (this, &FancyFileSelection::ok_click));
	get_cancel_button ()->clicked.connect (Gtk::Main::quit.slot ());

	show_all ();

	cancelled = true;
}

void FancyFileSelection::setFileType (const String ft)
{
	const char *ext = 0;
	filetype = ft;

	int num = sizeof (file_formats) / sizeof (file_formats[0]);
	for (int i = 0; i < num; ++i) {
		if (ft == file_formats[i].name) {
			ext = file_formats[i].extension;
			break;
		}
	}

	// Replace from the last dot onwards, unless no dot exists
	std::string fname = get_filename ();
	fname.erase (fname.rfind ('.'));
	fname += ext;
	set_filename (fname);
	complete (fname);
}
