/*
	gui.cc
*/

#include <gtk--/box.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
#include <gtk--/pixmap.h>
#include <gtk--/progressbar.h>
#include <gtk--/radiobutton.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/statusbar.h>
#include <gtk--/toolbar.h>
#include <gtk--/window.h>
#include <math.h>
#include "config.h"
#include "edge.h"
#include "graph.h"
#include "gui.h"
#include "lang.h"
#include "string.h"
#include "undo.h"
#include "vertex.h"


/* Toolbar pixmaps */
#include "edge_mode.xpm"
#include "vertex_mode.xpm"


static Gtk::Menu_Helpers::MenuElem gen_menu_head (const std::string &s,
							Gtk::Menu &menu)
{
	// create accel
	std::string accel = "<alt>";
	std::string::size_type idx = s.find ('_');
	if (s.length () >= idx)
		accel += s[idx + 1];
	else
		accel = "";

	return Gtk::Menu_Helpers::MenuElem (s, accel, menu);
}

Gtk::MenuBar *GUI::genMenuBar ()
{
	Gtk::MenuBar *mb = new Gtk::MenuBar ();

	using namespace Gtk::Menu_Helpers;

	/* File Menu */
	Gtk::Menu *menu_file = manage (new Gtk::Menu ());
	MenuList &lf = menu_file->items ();
	lf.push_back (MenuElem (_("_Load"), "<control>O",
			SigC::slot (this, &GUI::cb_File_Load)));
	lf.push_back (MenuElem (_("_Save"), "<control>S",
			SigC::slot (this, &GUI::cb_File_Save)));
	lf.push_back (MenuElem (_("_Print"), "<control>P",
			SigC::slot (this, &GUI::cb_File_Print)));
	lf.push_back (SeparatorElem ());
	lf.back ()->set_sensitive (false);
	lf.push_back (MenuElem (_("E_xit"), "<control>X",
					Gtk::Main::quit.slot ()));

	/* Edit Menu */
	Gtk::Menu *menu_edit = manage (new Gtk::Menu ());
	MenuList &le = menu_edit->items ();
	le.push_back (MenuElem (_("_Undo"), "<control>Z",
			SigC::slot (this, &GUI::cb_Edit_Undo)));
	edit_Undo = (Gtk::MenuItem *) le.back ();
	edit_Undo->set_sensitive (false);
	le.push_back (SeparatorElem ());
	le.back ()->set_sensitive (false);
	le.push_back (MenuElem (_("Select _All"), "<control>A",
			SigC::slot (this, &GUI::cb_Edit_SelectAll)));
	le.push_back (MenuElem (_("Select _None"),
			SigC::slot (this, &GUI::cb_Edit_SelectNone)));
	le.push_back (MenuElem (_("_Invert Selection (Vertices)"),
		SigC::slot (this, &GUI::cb_Edit_InvertSelectionVertices)));
	le.push_back (MenuElem (_("_Invert Selection (Edges)"),
		SigC::slot (this, &GUI::cb_Edit_InvertSelectionEdges)));
	le.push_back (MenuElem (_("_Invert Selection (All)"),
		SigC::slot (this, &GUI::cb_Edit_InvertSelectionAll)));

	/* View Menu */
	Gtk::Menu *menu_view = manage (new Gtk::Menu ());
	MenuList &lv = menu_view->items ();
	lv.push_back (CheckMenuElem (_("_Labels"),
				SigC::slot (this, &GUI::cb_View)));
	view_Labels = (Gtk::CheckMenuItem *) lv.back ();
	lv.push_back (CheckMenuElem (_("_Weights"),
				SigC::slot (this, &GUI::cb_View)));
	view_Weights = (Gtk::CheckMenuItem *) lv.back ();

	/* Graph Menu */
	Gtk::Menu *menu_graph = manage (new Gtk::Menu ());
	MenuList &lg = menu_graph->items ();
	lg.push_back (MenuElem (_("_Clear"), "<control>W",
			SigC::slot (this, &GUI::cb_Graph_Clear)));
	lg.push_back (MenuElem (_("C_omplement"),
			SigC::slot (this, &GUI::cb_Graph_Complement)));
	lg.push_back (MenuElem (_("_Line Graph"),
			SigC::slot (this, &GUI::cb_Graph_LineGraph)));
	lg.push_back (MenuElem (_("S_ubgraph"),
			SigC::slot (this, &GUI::cb_Graph_Subgraph)));
	lg.push_back (SeparatorElem ());
	lg.back ()->set_sensitive (false);

	/* Graph/Find submenu */
	Gtk::Menu *menu_graph_find = manage (new Gtk::Menu ());
	MenuList &lgf = menu_graph_find->items ();
	lgf.push_back (MenuElem (_("_Shortest Path"),
		SigC::slot (this, &GUI::cb_Graph_Find_ShortestPath)));
	lgf.push_back (MenuElem (_("_Breadth-First Search"),
		SigC::slot (this, &GUI::cb_Graph_Find_BFS)));
	lgf.push_back (MenuElem (_("_Depth-First Search"),
		SigC::slot (this, &GUI::cb_Graph_Find_DFS)));
	lgf.push_back (MenuElem (_("_Minimum Spanning Tree"),
		SigC::slot (this, &GUI::cb_Graph_Find_MST)));
	lg.push_back (MenuElem (_("_Find"), *menu_graph_find));

	/* Graph/Properties submenu */
	Gtk::Menu *menu_graph_prop = manage (new Gtk::Menu ());
	MenuList &lgp = menu_graph_prop->items ();
	lgp.push_back (MenuElem (_("_Connectivity"),
		SigC::slot (this, &GUI::cb_Graph_Properties_Connectivity)));
	lgp.push_back (MenuElem (_("_Eulericity"),
		SigC::slot (this, &GUI::cb_Graph_Properties_Eulericity)));
	lgp.push_back (MenuElem (_("_Hamiltonicity")));
	lgp.back ()->set_sensitive (false);
	lgp.push_back (MenuElem (_("_Planarity")));
	lgp.back ()->set_sensitive (false);
	lg.push_back (MenuElem (_("_Properties"), *menu_graph_prop));

	/* Graph/Statistics submenu */
	Gtk::Menu *menu_graph_stat = manage (new Gtk::Menu ());
	MenuList &lgs = menu_graph_stat->items ();
	lgs.push_back (MenuElem (_("_Adjacency Matrix"),
		SigC::slot (this, &GUI::cb_Graph_Statistics_AdjacencyMatrix)));
	lgs.push_back (MenuElem (_("_Degree Sequence"),
		SigC::slot (this, &GUI::cb_Graph_Statistics_DegreeSequence)));
	lgs.push_back (MenuElem (_("D_iameter"),
		SigC::slot (this, &GUI::cb_Graph_Statistics_Diameter)));
	lgs.push_back (MenuElem (_("_Girth")));
	lgs.back ()->set_sensitive (false);
	lgs.push_back (MenuElem (_("_Radius"),
		SigC::slot (this, &GUI::cb_Graph_Statistics_Radius)));
	lgs.push_back (SeparatorElem ());
	lgs.back ()->set_sensitive (false);
	lgs.push_back (MenuElem (_("_Chromatic Number"), SigC::slot (this,
			&GUI::cb_Graph_Statistics_ChromaticNumber)));
	lgs.push_back (MenuElem (_("C_hromatic Index"), SigC::slot (this,
			&GUI::cb_Graph_Statistics_ChromaticIndex)));
	lgs.push_back (MenuElem (_("Chromatic _Polynomial"), SigC::slot (this,
			&GUI::cb_Graph_Statistics_ChromaticPolynomial)));
	lg.push_back (MenuElem (_("_Statistics"), *menu_graph_stat));

	/* Prefab Menu */
	Gtk::Menu *menu_prefab = manage (new Gtk::Menu ());
	MenuList &lp = menu_prefab->items ();
	lp.push_back (MenuElem (_("Complete (_Kn)"),
			SigC::slot (this, &GUI::cb_Prefab_Complete)));
	lp.push_back (MenuElem (_("Complete _Bipartite (Kn,m)"),
		SigC::slot (this, &GUI::cb_Prefab_CompleteBipartite)));
	lp.push_back (MenuElem (_("Cycle (_Cn)"),
			SigC::slot (this, &GUI::cb_Prefab_Cycle)));
	lp.push_back (MenuElem (_("Gear (_Gn)"),
			SigC::slot (this, &GUI::cb_Prefab_Gear)));
	lp.push_back (MenuElem (_("Hanoi (_Hn)"),
			SigC::slot (this, &GUI::cb_Prefab_Hanoi)));
	lp.push_back (MenuElem (_("Ladder (_Ln)"),
			SigC::slot (this, &GUI::cb_Prefab_Ladder)));
	lp.push_back (MenuElem (_("Lattice (Ln,m)"),
			SigC::slot (this, &GUI::cb_Prefab_Lattice)));
	lp.push_back (MenuElem (_("Null (_Nn)"),
			SigC::slot (this, &GUI::cb_Prefab_Null)));
	lp.push_back (MenuElem (_("Star (_Sn)"),
			SigC::slot (this, &GUI::cb_Prefab_Star)));
	lp.push_back (MenuElem (_("Tree (_Tn)")));
	lp.back ()->set_sensitive (false);
	lp.push_back (MenuElem (_("Wheel (_Wn)"),
			SigC::slot (this, &GUI::cb_Prefab_Wheel)));
	lp.push_back (SeparatorElem ());
	lp.back ()->set_sensitive (false);
	lp.push_back (MenuElem (_("Petersen"),
			SigC::slot (this, &GUI::cb_Prefab_Petersen)));

	/* Prefab/Platonic submenu */
	Gtk::Menu *menu_prefab_platonic = manage (new Gtk::Menu ());
	MenuList &lpp = menu_prefab_platonic->items ();
	lpp.push_back (MenuElem (_("_Tetrahedral"),
		SigC::slot (this, &GUI::cb_Prefab_Platonic_Tetrahedral)));
	lpp.push_back (MenuElem (_("_Cubical"),
		SigC::slot (this, &GUI::cb_Prefab_Platonic_Cubical)));
	lpp.push_back (MenuElem (_("_Octahedral"),
		SigC::slot (this, &GUI::cb_Prefab_Platonic_Octahedral)));
	lpp.push_back (MenuElem (_("_Dodecahedral"),
		SigC::slot (this, &GUI::cb_Prefab_Platonic_Dodecahedral)));
	lpp.push_back (MenuElem (_("_Icosahedral"),
		SigC::slot (this, &GUI::cb_Prefab_Platonic_Icosahedral)));
	lp.push_back (MenuElem (_("_Platonic"), *menu_prefab_platonic));

	/* Help Menu */
	Gtk::Menu *menu_help = manage (new Gtk::Menu ());
	MenuList &lh = menu_help->items ();
	lh.push_back (MenuElem (_("_About"),
			SigC::slot (this, &GUI::cb_Help_About)));

	/* Menubar */
	MenuList &lb = mb->items ();
	lb.push_back (gen_menu_head (_("_File"), *menu_file));
	lb.push_back (gen_menu_head (_("_Edit"), *menu_edit));
	lb.push_back (gen_menu_head (_("_View"), *menu_view));
	lb.push_back (gen_menu_head (_("_Graph"), *menu_graph));
	lb.push_back (gen_menu_head (_("_Prefab"), *menu_prefab));
	lb.push_back (gen_menu_head (_("_Help"), *menu_help));
	lb.back ()->right_justify ();

	return mb;
}

void GUI::setUndoText (String description)
{
	using Gtk::Box_Helpers::BoxList;
	Gtk::Box *box = (Gtk::Box *) edit_Undo->get_child ();
	BoxList &kids = box->children ();
	Gtk::Label *txt = (Gtk::Label *) (*kids.begin ())->get_widget ();

	String s_txt (_("Undo"));
	if (description != "")
		s_txt += " (" + description + ")";
	txt->set_text (s_txt.std_str ());
}

int GUI::delete_event_impl (GdkEventAny *ev)
{
	Gtk::Main::quit ();
	return false;
}

int GUI::key_press_event_impl (GdkEventKey *ev)
{
	//std::cerr << "key_press_event_impl ("
	//	<< "ev->state = " << ev->state << ", "
	//	<< "ev->keyval = " << ev->keyval << ");\n";
	int mask = GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK;
	if (!(ev->state & mask) && (ev->keyval == GDK_Delete)) {
		// delete selected objects
		undoableAction ("Delete objects");	// TODO: make more accurate

		Edge *prevE, *e = graph->e_selected_head;
		while (e) {
			prevE = e;
			e = e->next;
			graph->remove (prevE);
		}

		Vertex *prevV, *v = graph->v_selected_head;
		while (v) {
			prevV = v;
			v = v->next;
			graph->remove (prevV);
		}

		canvas->redraw ();
		return false;
	}
	// FIXME: the following code works (i.e. the mode changes), but
	//		the toolbar buttons don't change!
	//if (!ev->state && (ev->keyval == ' ')) {
	//	cb_Change_Mode (editing_mode == VERTEX_MODE ?
	//					EDGE_MODE : VERTEX_MODE);
	//	return false;
	//}

	return Gtk::Window::key_press_event_impl (ev);
}

void GUI::realize_impl ()
{
	Gtk::Window::realize_impl ();

	view_Labels->set_active ();
}

GUI::GUI ()
{
	graph = new Graph ();


	set_default_size (500, 400);
	set_position (GTK_WIN_POS_CENTER);
	set_title ("GraphThing " GT_VERSION);


	/* Fill with a vbox */
	Gtk::VBox *vbox = manage (new Gtk::VBox (false, 0));
	add (*vbox);

	/* Menubar */
	Gtk::MenuBar *mb = manage (GUI::genMenuBar ());
	vbox->pack_start (*mb, false, false);

	/* Toolbar */
	toolbar = manage (new Gtk::Toolbar (
			GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS));
	toolbar->set_tooltips (true);
	toolbar->set_space_size (10);
	{
		using namespace Gtk::Toolbar_Helpers;

		toolbar->tools ().push_back (Space ());
		// Vertex/Edge Mode
		Gtk::RadioButton::Group gr;
		Gtk::Pixmap *vPix = manage (new Gtk::Pixmap (vertex_mode_xpm));
		Gtk::Pixmap *ePix = manage (new Gtk::Pixmap (edge_mode_xpm));
		vmode = new RadioElem (gr, *vPix,
			SigC::bind (
				SigC::slot (this, &GUI::cb_Change_Mode),
				VERTEX_MODE),
			"Change to vertex mode");
		toolbar->tools ().push_back (*vmode);
		emode = new RadioElem (gr, *ePix,
			SigC::bind (
				SigC::slot (this, &GUI::cb_Change_Mode),
				EDGE_MODE),
			"Change to edge mode");
		toolbar->tools ().push_back (*emode);

		Gtk::Main::timeout.connect (
			slot (this, &GUI::cb_Change_Mode_Hack), 500);
	}
	vbox->pack_start (*toolbar, false, false);

	/* Scrolled window with a drawing area */
	Gtk::ScrolledWindow *scwin = manage (new Gtk::ScrolledWindow ());
	scwin->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	vbox->pack_start (*scwin);

	canvas = manage (new Canvas (this, &graph));
	scwin->add_with_viewport (*canvas);

	/* Add a status bar and progress bar */
	Gtk::HBox *bottom_row = manage (new Gtk::HBox (false, 0));
	vbox->pack_end (*bottom_row, false, false);
	statusBar = manage (new Gtk::Statusbar ());
	bottom_row->pack_start (*statusBar, true, true);
	progressBar = manage (new Gtk::ProgressBar ());
	bottom_row->pack_end (*progressBar, false, false);

	cb_View ();

	show_all ();


	set_progress (-1);
	//statusBar->push (1, _("Ready."));
}

GUI::~GUI ()
{
	delete vmode;
	delete emode;

	delete graph;
}

void GUI::undoableAction (String description)
{
	UndoStep step (new Graph (*graph), description);
	undoStack.push (step);
	edit_Undo->set_sensitive (true);

	setUndoText (description);
}

void GUI::load_graph (const char *fname)
{
	bool res;

	Graph *g = graph->load (fname, res);
	if (g) {
		// TODO: Make this undoable?
		delete graph;
		graph = g;
	}

	canvas->redraw ();
}

void GUI::msg (const String &title, const String &s)
{
	MsgDialog m (title, s);

	Gtk::Main::run ();
}

gint GUI::quitter ()
{
	Gtk::Main::quit ();
	return 0;
}

void GUI::set_progress (double frac)
{
	static double last_frac = -1.0;

	if (frac < 0.0) {
		// "hide" progress bar
		progressBar->set_show_text (false);
		progressBar->hide ();
		last_frac = frac - 1;
	} else if (frac <= 1.0) {
		progressBar->set_percentage (frac);
		progressBar->set_format_string ("%p%% complete");
		progressBar->set_show_text (true);
		progressBar->show ();
	}

	// FIXME: KLUDGE! I couldn't work out any other way of making the
	//	progress meter update whilst we run, except by making this a
	//	multithreaded program - I don't want to do that until at least
	//	the 1.x series.
	if (fabs (frac - last_frac) > 0.01) {
		// Only update when we advance by at least 1%
		last_frac = frac;
		Gtk::Main::timeout.connect (slot (this, &GUI::quitter), 2);
		Gtk::Main::run ();
	}
}
