/*
	gui_aux.cc
*/


#include <gdk/gdkkeysyms.h>
#include <gtk--/adjustment.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/buttonbox.h>
#include <gtk--/drawingarea.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/menu.h>
#include <gtk--/optionmenu.h>
#include <gtk--/spinbutton.h>
#include <gtk--/window.h>
#include <math.h>
#include <sstream>
#include "config.h"
#include "edge.h"
#include "factory.h"
#include "graph.h"
#include "gui.h"
#include "lang.h"
#include "string.h"
#include "vertex.h"



int MsgDialog::delete_event_impl (GdkEventAny *ev)
{
	Gtk::Main::quit ();
	return false;
}

MsgDialog::MsgDialog (const String &title, const String &s)
{
	set_modal (true);
	set_policy (false, false, true);
	set_position (GTK_WIN_POS_CENTER);
	set_title (title.c_str ());

	/* Put about text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	get_vbox ()->add (*lbl);

	/* Put an OK button in the action area */
	Gtk::Button *butt = manage (new Gtk::Button (_("OK")));
	butt->set_usize (80, -1);
	butt->clicked.connect (Gtk::Main::quit.slot ());
	get_action_area ()->pack_start (*butt, false, false, 15);

	butt->grab_focus ();

	show_all ();
}

void ParamDialog::ok_click ()
{
	cancelled = false;
	get_data ();
	Gtk::Main::quit ();
}

int ParamDialog::delete_event_impl (GdkEventAny *ev)
{
	Gtk::Main::quit ();
	return false;
}

ParamDialog::ParamDialog (const String &title)
{
	set_modal (true);
	set_policy (false, false, true);
	set_position (GTK_WIN_POS_CENTER);
	set_title (title.c_str ());

	/* Content is handled by derived class */

	/* Put an OK and a Cancel button in the action area */
	Gtk::HButtonBox *hbb = manage (new Gtk::HButtonBox ());
	hbb->set_layout (GTK_BUTTONBOX_SPREAD);

	Gtk::Button *buttOK = manage (new Gtk::Button (_("OK")));
	buttOK->set_usize (80, -1);
	buttOK->clicked.connect (slot (this, &ParamDialog::ok_click));
	hbb->pack_start (*buttOK, false, false, 15);
	Gtk::Button *buttCancel = manage (new Gtk::Button (_("Cancel")));
	buttCancel->set_usize (80, -1);
	buttCancel->clicked.connect (Gtk::Main::quit.slot ());
	hbb->pack_start (*buttCancel, false, false, 15);

	get_action_area ()->pack_start (*hbb, false, false, 0);

	buttOK->grab_focus ();

	cancelled = true;
}

void IntParamDialog::get_data ()
{
	sp->update ();
	param = sp->get_value_as_int ();
}

IntParamDialog::IntParamDialog (const String &s, int min, int initial, int max)
							: ParamDialog ()
{
	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the spin button */
	Gtk::Adjustment *adj = manage
		(new Gtk::Adjustment (initial, min, max, 1, 5, 5));
	sp = manage (new Gtk::SpinButton (*adj, 1, 0));
	sp->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	sp->set_numeric (true);
	sp->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp, true, true, 10);

	get_vbox ()->add (*hbox);
	show_all ();

	sp->select_region (0, -1);
	sp->grab_focus ();
}

IntParamDialog::IntParamDialog (const String &title, const String &s,
		int min, int initial, int max) : ParamDialog (title)
{
	/* FIXME: Yes, this is a duplicate of the non-title version of
	 * this constructor - somehow things didn't work with a simple:
	 *	IntParamDialog (s, min, initial, max).
	 */

	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the spin button */
	Gtk::Adjustment *adj = manage
		(new Gtk::Adjustment (initial, min, max, 1, 5, 5));
	sp = manage (new Gtk::SpinButton (*adj, 1, 0));
	sp->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	sp->set_numeric (true);
	sp->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp, true, true, 10);

	get_vbox ()->add (*hbox);
	show_all ();

	sp->select_region (0, -1);
	sp->grab_focus ();
}

void IntIntParamDialog::get_data ()
{
	sp1->update ();
	sp2->update ();
	param1 = sp1->get_value_as_int ();
	param2 = sp2->get_value_as_int ();
}

IntIntParamDialog::IntIntParamDialog (const String &s, int min1,
		int initial1, int min2, int initial2) : ParamDialog ()
{
	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the spin buttons (FIXME: upper bound of 50?) */
	Gtk::Adjustment *adj1 = manage
		(new Gtk::Adjustment (initial1, min1, 50, 1, 5, 5));
	sp1 = manage (new Gtk::SpinButton (*adj1, 1, 0));
	sp1->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	sp1->set_numeric (true);
	sp1->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp1, true, true, 10);

	Gtk::Adjustment *adj2 = manage
		(new Gtk::Adjustment (initial2, min2, 50, 1, 5, 5));
	sp2 = manage (new Gtk::SpinButton (*adj2, 1, 0));
	sp2->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	sp2->set_numeric (true);
	sp2->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp2, true, true, 10);

	get_vbox ()->add (*hbox);
	show_all ();

	/* sp1->select_region (0, -1); */
	sp1->grab_focus ();
}

void StringParamDialog::get_data ()
{
	param = String (entry->get_text ());
}

StringParamDialog::StringParamDialog (const String &s, const String &initial)
							: ParamDialog ()
{
	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the edit box */
	entry = manage (new Gtk::Entry (50));
	entry->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	hbox->pack_start (*entry, false, false, 10);

	get_vbox ()->add (*hbox);
	show_all ();

	entry->set_text (initial.std_str ());
	entry->select_region (0, -1);
	entry->grab_focus ();
}

StringParamDialog::StringParamDialog (const String &title, const String &s,
			const String &initial) : ParamDialog (title)
{
	/* FIXME: Yes, this is a duplicate of the non-title version of
	 * this constructor - somehow things didn't work with a simple:
	 *	StringParamDialog (s, initial).
	 */

	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (s.std_str ()));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the edit box */
	entry = manage (new Gtk::Entry (50));
	entry->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	hbox->pack_start (*entry, false, false, 10);

	get_vbox ()->add (*hbox);
	show_all ();

	entry->set_text (initial.std_str ());
	entry->select_region (0, -1);
	entry->grab_focus ();
}

void EdgeParamDialog::get_data ()
{
	sp->update ();
	weight = sp->get_value_as_int ();
}

EdgeParamDialog::EdgeParamDialog (const Edge *e)
					: ParamDialog (_("Edge Properties"))
{
	Gtk::HBox *hbox = manage (new Gtk::HBox ());

	/* Put text in place */
	Gtk::Label *lbl = manage (new Gtk::Label (_("Edge weight:")));
	lbl->set_padding (30, 30);
	hbox->pack_start (*lbl, true, true, 10);

	/* ... and the spin button */
	Gtk::Adjustment *adj = manage
		(new Gtk::Adjustment (e->weight, 0, max_edge_weight, 1, 5, 5));
	sp = manage (new Gtk::SpinButton (*adj, 1, 0));
	sp->activate.connect (SigC::slot (this, &ParamDialog::ok_click));
	sp->set_numeric (true);
	sp->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp, true, true, 10);

	get_vbox ()->pack_start (*hbox, false, false, 10);

	// ... and the option menu (with vertex labels)
	Gtk::HBox *hbox2 = manage (new Gtk::HBox ());

	Gtk::Label *lblV = manage (new Gtk::Label (e->v->label.std_str ()));
	hbox2->pack_start (*lblV, true, true, 5);

	using namespace Gtk::Menu_Helpers;
	Gtk::OptionMenu *dir = manage (new Gtk::OptionMenu ());
	Gtk::Menu *opt_m = manage (new Gtk::Menu ());
	MenuList &lo = opt_m->items ();
	lo.push_back (MenuElem ("<---",
			SigC::slot (this, &EdgeParamDialog::setLeft)));
	lo.push_back (MenuElem ("----",
			SigC::slot (this, &EdgeParamDialog::setNeither)));
	lo.push_back (MenuElem ("--->",
			SigC::slot (this, &EdgeParamDialog::setRight)));
	direction = (e->directed ? 1 : 0);
	opt_m->set_active (direction + 1);
	dir->set_menu (opt_m);
	hbox2->pack_start (*dir, false, false, 10);

	Gtk::Label *lblW = manage (new Gtk::Label (e->w->label.std_str ()));
	hbox2->pack_start (*lblW, true, true, 5);

	get_vbox ()->pack_end (*hbox2, false, false, 10);

	show_all ();

	sp->select_region (0, -1);
	sp->grab_focus ();
}

void Canvas::draw (Edge *e)
{
	double dx, dy, r = Graph::vertex_radius;
	double theta;

	theta = atan2 (e->w->y - e->v->y, e->w->x - e->v->x);
	dx = r * cos (theta) / 2;
	dy = r * sin (theta) / 2;

	if (e->selected)
		gc.set_foreground (red);

	gc.set_line_attributes (1, GDK_LINE_SOLID, GDK_CAP_BUTT,
							GDK_JOIN_MITER);
	if (e->v->y == e->w->y)		// horizontal line (dy = 0)
		dy = 0;
	else if (e->v->x == e->w->x)	// vertical line (dx = 0)
		dx = 0;

	win.draw_line (gc, int (e->v->x + dx), int (e->v->y + dy),
				int (e->w->x - dx), int (e->w->y - dy));

	if (e->directed) {
		GdkPoint tri[3];
		double scale = 1.5;
		tri[0].x = (int) (e->w->x - dx);
		tri[0].y = (int) (e->w->y - dy);
		tri[1].x = (int) (e->w->x - dx - scale * (dx - dy));
		tri[1].y = (int) (e->w->y - dy - scale * (dx + dy));
		tri[2].x = (int) (e->w->x - dx - scale * (dx + dy));
		tri[2].y = (int) (e->w->y - dy - scale * (dy - dx));
		win.draw_polygon (gc, true, tri, 3);
	}

	if (do_weights) {
		std::ostringstream buf;
		double rx, ry;

		buf << e->weight;
		rx = (e->v->x + e->w->x) / 2 + dy;
		ry = (e->v->y + e->w->y) / 2 - dx;
		if (theta < 0)
			rx += (int) (normal_font.string_width (buf.str ())
							* sin (theta));

		if (fabs (theta) > M_PI_2)
			ry -= 2 * dx;

		win.draw_string (normal_font, gc, int (rx), int (ry),
								buf.str ());
	}

	if (e->selected)
		gc.set_foreground (black);
}

void Canvas::draw (Vertex *v)
{
	int r = Graph::vertex_radius;

	if (v->selected) {
		unsigned int idx = v->selection_colour %
						selection_colours.size ();
		gc.set_foreground (selection_colours[idx]);
	}

	win.draw_arc (gc, true, v->x - r / 2, v->y - r / 2, r, r, 0,
								64 * 360);

	if (do_labels)
		win.draw_string (bold_font, gc, v->x + r, v->y - r,
						v->label.c_str ());

	if (v->selected)
		gc.set_foreground (black);
}

void Canvas::cb_Properties ()
{
	Edge *e;
	Graph *g = *gg;
	Vertex *v;

	v = g->v_selected_head;
	e = g->e_selected_head;

	if (v && !e) {
		/* Vertex Properties */

		StringParamDialog spd (_("Vertex Properties"),
					_("Label:"), v->label);
		Gtk::Main::run ();
		if (spd.cancelled)
			return;

		/* Handle duplicate labels */
		while (1) {
			Vertex *f = g->find (spd.param);
			if (!f || (f == v))
				break;
			spd.param += "-dup";
		}

		if (spd.param != v->label) {
			gui->undoableAction ("Rename vertex");
			g->rename (v, spd.param);
			redraw ();
		}
	} else if (e) {
		/* Edge Properties */

		EdgeParamDialog epd (e);
		Gtk::Main::run ();
		if (epd.cancelled)
			return;

		int curr_dir = e->directed ? 1 : 0;
		if ((epd.weight == e->weight) && (curr_dir == epd.direction))
			return;
		if (epd.weight == e->weight)
			gui->undoableAction ("Change edge direction");
		else if (curr_dir == epd.direction)
			gui->undoableAction ("Change edge weight");
		else
			gui->undoableAction ("Change edge weight and direction");
		e->weight = epd.weight;
		if (epd.direction == 0)
			e->directed = false;
		else if (epd.direction == 1)
			e->directed = true;
		else {
			e->directed = true;
			Vertex *tmp = e->v;
			e->v = e->w;
			e->w = tmp;
		}
		redraw ();
	}
}

int Canvas::button_press_event_impl (GdkEventButton *ev)
{
	Graph *g = *gg;
	Vertex *v;

	ev->state &= GDK_SHIFT_MASK;

	/* Assumptions: none */

	if ((ev->button != 1) && (ev->button != 3))
		return false;	// ignore non-left and non-right mouse buttons
	if (ev->type == GDK_BUTTON_RELEASE)
		return false;	// ignore button releases

	bool left_click = (ev->button == 1);

	v = g->find (int (ev->x), int (ev->y));

	/* Assumptions: [lr]-click */

	if (vertex_mode && left_click) {
		if (!(ev->state & GDK_SHIFT_MASK)) {
			// no shift key
			if (ev->type == GDK_BUTTON_PRESS) {
				g->unselect_all ();
				if (v)
					g->select (v);
				redraw ();
				return false;
			} else if (ev->type == GDK_2BUTTON_PRESS) {
				if (v)
					cb_Properties ();
				else {
					gui->undoableAction ("Add vertex");
					g->add (new Vertex ("",
						int (ev->x), int (ev->y)));
					redraw ();
				}
				return false;
			}
		} else {
			// shift key
			if (ev->type == GDK_BUTTON_PRESS) {
				if (v) {
					g->select (v);
					redraw ();
				}
			}
			return false;
		}
	} else if (vertex_mode && !left_click)
		return false;	// do nothing

	// edge mode
	if (!v) {
		Edge *e = g->findE (int (ev->x), int (ev->y));
		if (!e) {
			if (left_click) {
				g->unselect_all ();
				redraw ();
			}
			return false;
		}
		// clicking on an edge
		if (ev->type == GDK_BUTTON_PRESS) {
			if (left_click)
				g->select (e);
			else
				e->cycle_orientations ();
			redraw ();
		} else if (ev->type == GDK_2BUTTON_PRESS) {
			if (left_click)
				cb_Properties ();
		}
		return false;
	}
	if (!left_click)
		return false;
	if (!v->selected) {
		Vertex *valt = g->v_selected_head;
		if (!valt) {
			g->select (v);
			redraw ();
			return false;
		}
		if (!g->are_adjacent (valt, v)) {
			gui->undoableAction ("Add edge");
			g->add (new Edge (valt, v));
		}
		g->unselect_all ();
		g->select (v);
		redraw ();
		return false;
	}
	g->unselect (v);
	redraw ();


	return false;
}

int Canvas::expose_event_impl (GdkEventExpose *ev)
{
	redraw ();

	return true;
}

int Canvas::motion_notify_event_impl (GdkEventMotion *ev)
{
	Graph *g = *gg;
	Vertex *v;
	int evx = (int) ev->x, evy = (int) ev->y;


	if (!(ev->state & GDK_BUTTON1_MASK)) {
		/* Button not down => nothing to do */
		motion_last_x = -1;
		motion_last_y = -1;
		return false;
	}

	if (motion_last_x == -1) {
		/* First motion event; record start only */
		motion_last_x = evx;
		motion_last_y = evy;
		return false;
	}

	/* Move stuff */
	for (v = g->v_selected_head; v; v = v->next) {
		v->x += (evx - motion_last_x);
		v->y += (evy - motion_last_y);
	}

	motion_last_x = evx;
	motion_last_y = evy;

	redraw ();
	return false;
}

void Canvas::realize_impl ()
{
	Gtk::DrawingArea::realize_impl ();

	win = get_window ();
	gc.create (win);
	win.set_background (white);
	win.clear ();
	gc.set_foreground (black);

	win.set_background (white);
}

void Canvas::size_allocate_impl (GtkAllocation *al)
{
	Gtk::DrawingArea::size_allocate_impl (al);

	Factory::width = al->width;
	Factory::height = al->height;
}

Canvas::Canvas (GUI *the_gui, Graph **g) : gui (the_gui), gg (g)
{
	set_events (	GDK_POINTER_MOTION_MASK |
			GDK_BUTTON_PRESS_MASK |
			GDK_KEY_PRESS_MASK );


	cmap = get_default_colormap ();

	white = Gdk_Color ("white");
	cmap.alloc (white);

	// Selection colours
	const char *cols[] = {
		"red", "black", "blue", "purple", "brown", "orange",
		"dark green", "cyan"
	};
	for (unsigned int i = 0; i < (sizeof (cols) / sizeof (cols[0])); ++i) {
		Gdk_Color col = Gdk_Color (cols[i]);
		cmap.alloc (col);
		selection_colours.push_back (col);
	}
	red = selection_colours[0];
	black = selection_colours[1];

	bold_font = Gdk_Font
		("-*-helvetica-bold-r-normal--12-*-*-*-p-*-*-*");
	normal_font = Gdk_Font
		("-*-helvetica-medium-r-normal--12-*-*-*-p-*-*-*");

	do_labels = true;
	do_weights = true;
	motion_last_x = -1;
	motion_last_y = -1;
	size_x = 0;
	size_y = 0;
}

void Canvas::redraw ()
{
	int w, h;
	Graph::e_const_iterator eit;
	Graph::v_const_iterator vit;
	Graph *g = *gg;
	int maxx, maxy;

	win.get_size (w, h);
	win.clear ();

	gc.set_foreground (black);
	maxx = 0;
	maxy = 0;
	for (vit = g->v_begin (); vit != g->v_end (); ++vit) {
		draw (*vit);
		if ((*vit)->x >= maxx)
			maxx = (*vit)->x;
		if ((*vit)->y >= maxy)
			maxy = (*vit)->y;
	}
	for (eit = g->e_begin (); eit != g->e_end (); ++eit)
		draw (*eit);

	if (((maxx + 20) > size_x) || ((maxy + 20) > size_y)) {
		size_x = maxx + 20;
		size_y = maxy + 20;
		size (size_x, size_y);
	}
}

void Canvas::set_param (bool labels, bool weights)
{
	do_labels = labels;
	do_weights = weights;

	if (is_realized ())
		redraw ();
}
