/*
	gui_matrix.cc
*/

#include <gtk--/adjustment.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/dialog.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/spinbutton.h>
#include <gtk--/table.h>
#include <sstream>
#include <vector>
#include "graph.h"
#include "gui.h"
#include "lang.h"
#include "matrix.h"
#include "string.h"
#include "vertex.h"


#include <iostream>
void MatrixDialog::update_view ()
{
	int exp = sp->get_value_as_int (), sz = matrices.size ();
	unsigned int i, j;

	// Compute any powers we don't already have cached
	while (sz <= exp) {
		matrices.push_back (matrices[sz - 1] * matrices[1]);
		sz = matrices.size ();
	}

	Matrix &mat = matrices[exp];

	// Clear existing table
	get_vbox ()->remove (*tbl);

	// Recreate table
	tbl = manage (new Gtk::Table (mat.rows () + 1, mat.columns () + 1,
									true));
	get_vbox ()->pack_start (*tbl, true, true, 20);

	// Add row and column labels
	for (i = 0; i < mat.rows (); ++i) {
		Gtk::Label *lbl = manage (
				new Gtk::Label (labels[i].std_str ()));
		tbl->attach (*lbl, i + 1, i + 2, 0, 1);
		Gtk::Label *lbl2 = manage (
				new Gtk::Label (labels[i].std_str ()));
		tbl->attach (*lbl2, 0, 1, i + 1, i + 2);
	}

	// Add contents
	for (j = 0; j < mat.rows (); ++j)
		for (i = 0; i < mat.columns (); ++i) {
			std::ostringstream buf;
			buf << mat (i, j);
			Gtk::Label *lbl = manage (
					new Gtk::Label (buf.str ()));
			lbl->set_padding (5, 5);
			tbl->attach (*lbl, j + 1, j + 2, i + 1, i + 2);
		}

	show_all ();
}

int MatrixDialog::delete_event_impl (GdkEventAny *ev)
{
	Gtk::Main::quit ();
	return false;
}

MatrixDialog::MatrixDialog (const String &title, const Matrix &_mat,
							const Graph &g)
{
	unsigned int i;

	// Create identity matrix (A^0)
	Matrix id = _mat - _mat;	// zero matrix
	for (i = 0; i < id.rows (); ++i)
		id(i, i) = 1;

	/* TODO: Check that mat.rows () == mat.columns () */
	matrices.push_back (id);
	matrices.push_back (_mat);

	set_modal (true);
	set_policy (true, true, true);
	set_position (GTK_WIN_POS_CENTER);
	set_title (title.std_str ());

	/* Create table */
	for (i = 0; i < _mat.rows (); ++i)
		labels.push_back (g[i]->label);
	tbl = manage (new Gtk::Table (_mat.rows () + 1, _mat.columns () + 1,
									true));
	get_vbox ()->pack_start (*tbl, true, true, 20);

	// Put a label + spinbutton below table
	Gtk::HBox *hbox = manage (new Gtk::HBox ());
	Gtk::Label *lbl = manage (new Gtk::Label (_("Exponent:")));
	lbl->set_padding (10, 10);
	hbox->pack_start (*lbl, false, false, 10);
	Gtk::Adjustment *adj = manage
		(new Gtk::Adjustment (1, 0, 10, 1, 1, 1));
	sp = manage (new Gtk::SpinButton (*adj, 1, 0));
	sp->activate.connect (Gtk::Main::quit.slot ());
	sp->changed.connect (SigC::slot (this, &MatrixDialog::update_view));
	sp->set_numeric (true);
	sp->set_update_policy (GTK_UPDATE_IF_VALID);
	hbox->pack_start (*sp, false, false, 10);
	get_vbox ()->pack_end (*hbox, false, false);


	/* Put an OK button in the action area */
	Gtk::Button *butt = manage (new Gtk::Button (_("OK")));
	butt->set_usize (80, -1);
	butt->clicked.connect (Gtk::Main::quit.slot ());
	get_action_area ()->pack_start (*butt, false, false, 15);

	butt->grab_focus ();

	update_view ();

	show_all ();
}
