/*
	lang.h
*/

#ifndef __LANG_H__
#define __LANG_H__

#include "config.h"

#ifdef ENABLE_NLS
#include <map>
#include "string.h"

typedef enum {
	Catalan, Danish, Dutch, English, Esperanto, Finnish, French, German,
	Greek, Italian, Norwegian, Polish, Portuguese, Romanian, Spanish,
	Swedish, SwedishChef
} Language;


// A phrase (in English), and all its translations
// TODO: Make this a base class, and derive classes for the different types
class PhraseBlock
{
public:
	typedef enum { regular, concat, literal, replace } type_t;

private:
	String english;
	type_t type;

	// for 'regular' type
	std::map<Language, String> translations;

	// for 'concat' type
	String left, right;

	// for 'replace' type
	String base, substr, substr_new;

public:
	PhraseBlock (String eng, type_t type = regular);
	~PhraseBlock ();

	void set_type (type_t type);
	void set_param (int id, String param);

	void add (Language lang, String translation);
	const String lookup (const Language lang) const;

	int verify_phrase () const;
};

typedef struct {
	char *english, *other;
} raw_trans;

class Translator
{
private:
	std::map<const String, PhraseBlock *> phrases;

	Language default_lang;

	void split_lang_spec (const char *spec, String &LL, String &CC) const;

public:
	static int warn_untranslated;

	Translator (int debug = 0);
	~Translator ();

	void init ();
	static String get_language_name (Language lang);

	void verify_phrases () const;

	void guess_language (bool ignore_cc = false);
	void set_language (Language lang);
	Language get_language () const;

	void add_phrase (PhraseBlock *phr);

	const String lookup (Language lang, const String s) const;
	const String lookup (const String s) const;
	const char *lookup (const char *str) const;

	// avoid calling this, unless necessary
	PhraseBlock *lookup_phr (const String s) const;
};


extern Translator *translator;	// defined in main.cc

#define _(String)	(translator->lookup (String))

#else	/* ! ENABLE_NLS */

#define _(String)	(String)

#endif

#endif	/* __LANG_H__ */
