/*
	string.cc
*/

#include <iostream>
#include <string>
#include <string.h>
#include "string.h"


String::StringRep::StringRep ()
{
	len = 0;
	size = 8;
	ref = 0;

	data = new char[size];
	data[0] = 0;
}

String::StringRep::StringRep (unsigned int n)
{
	len = 0;
	size = (n + 8) & ~7;
	ref = 0;

	data = new char[size];
	data[0] = 0;
}

String::StringRep::StringRep (const char *s)
{
	len = strlen (s);
	size = (len + 8) & ~7;
	ref = 0;

	data = new char[size];
	strcpy (data, s);
}

String::StringRep::StringRep (const std::string &s)
{
	len = s.size ();
	size = (len + 8) & ~7;
	ref = 0;

	data = new char[size];
	strcpy (data, s.c_str ());
}

String::StringRep::~StringRep ()
{
	delete[] data;
}

void String::unref ()
{
	if (--rep->ref < 1)
		delete rep;
}

String::String ()
{
	rep = new StringRep ();
	++rep->ref;
}

String::String (const char *s)
{
	rep = new StringRep (s);
	++rep->ref;
}

String::String (const std::string &s)
{
	rep = new StringRep (s);
	++rep->ref;
}

String::String (const String &s)
{
	rep = s.rep;
	++rep->ref;
}

String::~String ()
{
	unref ();
}

int String::length () const
{
	return rep->len;
}

String &String::append (const char *s)
{
	int len = strlen (s);

	if (rep->ref > 1) {
		const StringRep *old = rep;
		unref ();
		rep = new StringRep (old->len + len + 1);
		++rep->ref;
		strcpy (rep->data, old->data);
		rep->len = old->len;
	}

	if ((rep->len + len + 1) > rep->size) {
		/* Reallocate */
		char *old = rep->data;
		rep->size = (rep->len + len + 9) & ~7;
		rep->data = new char[rep->size];
		strcpy (rep->data, old);
		delete[] old;
	}

	strcpy (&rep->data[rep->len], s);
	rep->len += len;

	return *this;
}

String &String::append (const String &s)
{
	return append (s.rep->data);
}

int String::compare (const char *s) const
{
	const char *a;

	a = rep->data;

	while (*a && *s) {
		if (*a < *s)
			return -1;
		if (*a > *s)
			return 1;
		++a;
		++s;
	}

	if (*a)
		return 1;
	if (*s)
		return -1;
	return 0;
}

int String::compare (const String &s) const
{
	if (rep == s.rep)
		return 0;

	return compare (s.rep->data);
}

const char *String::c_str () const
{
	return rep->data;
}

std::string String::std_str () const
{
	return std::string (rep->data);
}

bool String::empty () const
{
	return (rep->len == 0);
}

String &String::operator= (const String &other)
{
	if (this == &other)
		return *this;

	unref ();

	rep = other.rep;
	++rep->ref;

	return *this;
}

String &String::operator= (const char *s)
{
	unref ();

	rep = new StringRep (s);
	++rep->ref;

	return *this;
}

bool String::operator== (const char *s) const
{
	return (compare (s) == 0);
}

bool String::operator== (const String &s) const
{
	return (compare (s) == 0);
}

bool String::operator< (const String &s) const
{
	return (compare (s) < 0);
}

bool String::operator!= (const char *s) const
{
	return (compare (s) != 0);
}

bool String::operator!= (const String &s) const
{
	return (compare (s) != 0);
}

String &String::operator+= (const char *s)
{
	return append (s);
}

String &String::operator+= (const String &s)
{
	return append (s);
}

char &String::operator[] (unsigned int i)
{
	/* TODO: out-of-bounds detection */

	if (rep->ref > 1) {
		char *old = rep->data;
		unref ();
		rep = new StringRep (old);
	}

	return rep->data[i];
}

const char &String::operator[] (unsigned int i) const
{
	/* TODO: out-of-bounds detection */

	return rep->data[i];
}

std::ostream &operator<< (std::ostream &o, const String &s)
{
	return (o << s.rep->data);
}

String operator+ (const String &s1, const String &s2)
{
	String s = s1;
	s += s2;
	return s;
}
